/* showDesktop.js
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

/* exported getExtension */

const Gettext = imports.gettext;

const { GObject, Shell, St } = imports.gi;

const ExtensionUtils = imports.misc.extensionUtils;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;

const Me = ExtensionUtils.getCurrentExtension();

const _ = Gettext.domain(Me.metadata['gettext-domain']).gettext;

const ShowDesktopButton = GObject.registerClass(
class ShowDesktopButton extends PanelMenu.Button {
    _init() {
        super._init(0.0, _("Show Desktop"), true);
        this._hbox = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
        this._icon = new St.Icon({ icon_name: 'video-display-symbolic', style_class: 'system-status-icon' });
        this._hbox.add_child(this._icon);
        this.add_child(this._hbox);
        this.connect('button-press-event', this._onButtonPressEvent.bind(this));
        this._windows = [];
    }

    _onButtonPressEvent() {
        let tracker = Shell.WindowTracker.get_default();
        let workspace = global.workspace_manager.get_active_workspace();
        let workspaceIndex = workspace.index();
        let workspaceWindows = workspace.list_windows();
        if (this._windows[workspaceIndex]) {
            workspaceWindows.forEach(window => {
                if (tracker.get_window_app(window)) {
                    let restore = true;
                    for (let i = 0; i < this._windows[workspaceIndex].length; i++) {
                        if (window === this._windows[workspaceIndex][i]) {
                            restore = false;
                            break;
                        }
                    }
                    if (restore) window.unminimize();
                }
            });
            this._windows[workspaceIndex] = null;
        } else {
            this._windows[workspaceIndex] = [];
            workspaceWindows.forEach(window => {
                if (tracker.get_window_app(window)) {
                    let canClose = window.can_close ? window.can_close() : true;
                    if (canClose && !window.minimized) {
                        window.minimize();
                    } else {
                        this._windows[workspaceIndex].push(window);
                    }
                }
            });
        }
        Main.overview.hide();
    }
});

function getExtension() {
    return Main.panel.addToStatusArea('PanelManagerShowDesktop', new ShowDesktopButton(), 1, 'left');
}
